@@ -2,15 +2,28 @@ use strict;
 
 use Tangram;
 
+use Tangram::RawDate;
+use Tangram::RawTime;
+use Tangram::RawDateTime;
+
 package Springfield;
 
-use vars qw( $schema );
+use vars qw( $schema @ISA @EXPORT );
+
+require Exporter;
+@ISA = qw( Exporter );
+@EXPORT = qw( optional_tests $schema );
 
 $schema = Tangram::Schema->new( {
 
    #set_id => sub { my ($obj, $id) = @_; $obj->{id} = $id },
    #get_id => sub { shift()->{id} },
 
+   sql =>
+   {
+	   cid_size => 2
+   },
+								  
    classes =>
    {
       Person =>
@@ -20,138 +33,158 @@ $schema = Tangram::Schema->new( {
 
       NaturalPerson =>
       {
-         bases =>
-            [ qw( Person ) ],
+	   table => 'NP',
 
-         fields =>
-         {
-            string =>
-	    {
-		firstName => { sql => 'VARCHAR(40)' },
-		name => undef,
-	    },
+	   bases => [ qw( Person ) ],
 
-            int =>
-               [ qw( age ) ],
+	   fields =>
+	   {
+		string =>
+		{
+		 firstName => { sql => 'VARCHAR(40)' },
+		 name => undef,
+		},
 
-            ref =>
-               [ qw( partner ) ],
+		int => [ qw( age ) ],
 
-            array =>
-            {
-               a_children =>
-               {
-                  class => 'NaturalPerson',
-                  table => 'a_children',
-               }
-            },
+		ref =>
+		{
+		 partner => undef,
+		 credit => { aggreg => 1 },
+		},
 
-            hash =>
-            {
-               h_opinions =>
-               {
-                  class => 'Opinion',
-                  table => 'h_opinions',
-               }
-            },
+		#rawdate => [ qw( birthDate ) ],
+		#rawtime => [ qw( birthTime ) ],
+		rawdatetime => [ qw( birth ) ],
 
-            iarray =>
-            {
-               ia_children =>
-               {
-                  class => 'NaturalPerson',
-                  coll => 'ia_ref',
-                  slot => 'ia_slot',
-                  back => 'ia_parent'
-               }
-            },
+		array =>
+		{
+		 a_children =>
+		 {
+		  class => 'NaturalPerson',
+		  table => 'a_children',
+		  aggreg => 1,
+		 }
+		},
 
-            set =>
-            {
-               s_children =>
-               {
-                  class => 'NaturalPerson',
-                  table => 's_children',
-               }
-            },
+		hash =>
+		{
+		 h_opinions =>
+		 {
+		  class => 'Opinion',
+		  table => 'h_opinions',
+		 }
+		},
 
-            iset =>
-            {
-               is_children =>
-               {
-                  class => 'NaturalPerson',
-                  coll => 'is_ref',
-                  slot => 'is_slot',
-                  back => 'is_parent'
-               }
-            },
-         },
+		iarray =>
+		{
+		 ia_children =>
+		 {
+		  class => 'NaturalPerson',
+		  coll => 'ia_ref',
+		  slot => 'ia_slot',
+		  back => 'ia_parent',
+		  aggreg => 1,
+		 }
+		},
+
+		set =>
+		{
+		 s_children =>
+		 {
+		  class => 'NaturalPerson',
+		  table => 's_children',
+		  aggreg => 1,
+		 }
+		},
+
+		iset =>
+		{
+		 is_children =>
+		 {
+		  class => 'NaturalPerson',
+		  coll => 'is_ref',
+		  slot => 'is_slot',
+		  back => 'is_parent',
+		  aggreg => 1,
+		 }
+		},
+	   },
       },
 
-      Opinion =>
-      {
-         fields =>
-         {
-            string =>
-               [ qw( statement ) ],
-         },
-      },
+	Opinion =>
+	{
+	 fields =>
+	 {
+	  string => [ qw( statement ) ],
+	 },
+	},
 
-      LegalPerson =>
-      {
-         bases =>
-            [ qw( Person ) ],
+	LegalPerson =>
+	{
+	 bases => [ qw( Person ) ],
 
-         fields =>
-         {
-            string =>
-               [ qw( name ) ],
+	 fields =>
+	 {
+	  string =>
+	  [ qw( name ) ],
 
-            ref =>
-	    {		
-		manager => { null => 1 }
-	    },
-         },
-      },
+	  ref =>
+	  {		
+	   manager => { null => 0 }
+	  },
+	 },
+	},
 
-      EcologicalRisk =>
-      {
-         abstract => 1,
+	EcologicalRisk =>
+	{
+	 abstract => 1,
 
-         fields =>
-         {
-            int => [ qw( curies ) ],
-         },
-      },
+	 fields =>
+	 {
+	  int => [ qw( curies ) ],
+	 },
+	},
    
-      NuclearPlant =>
-      {
-         bases => [ qw( LegalPerson EcologicalRisk ) ],
+	NuclearPlant =>
+	{
+	 bases => [ qw( LegalPerson EcologicalRisk ) ],
 
-         fields =>
-         {
-            array =>
-            {
-               employees =>
-               {
-                  class => 'NaturalPerson',
-                  table => 'employees'
-               }
-            },
-         },
-      },
+	 fields =>
+	 {
+	  array =>
+	  {
+	   employees =>
+	   {
+		class => 'NaturalPerson',
+		table => 'employees'
+	   }
+	  },
+	 },
+	},
+
+	Credit =>
+	{
+	 fields =>
+	 {
+	  #int => { limit => { col => '_limit' } },
+	  int => { limit => '_limit' },
+	 }
+	},
 
    } } );
 
-my ($cs, $user, $passwd);
+use vars qw( $cs $user $passwd);
 
 {
    local $/;
+
+   my $config = $ENV{TANGRAM_CONFIG} || 'CONFIG';
    
-   open CONFIG, 'CONFIG'
-   or open CONFIG, 't/CONFIG'
-   or open CONFIG, '../t/CONFIG'
-   or die "Cannot open 't/CONFIG', reason: $!";
+   open CONFIG, $config
+   or open CONFIG, "t/$config"
+   or open CONFIG, "../t/$config"
+   or die "Cannot open t/$config, reason: $!";
 
    ($cs, $user, $passwd) = split "\n", <CONFIG>;
 }
@@ -231,6 +264,20 @@ sub tx_tests
 	}
 }
 
+sub optional_tests
+{
+	my ($what, $proceed, $tests) = @_;
+
+	unless ($proceed)
+	{
+		print STDERR "tests $test-", $test + $tests - 1,
+			" ($what) skipped on this platform ";
+		test(1) while $tests--;
+	}
+
+	return $proceed;
+}
+
 #use Data::Dumper;
 #print Dumper $schema;
 #deploy;
@@ -311,4 +358,7 @@ use vars qw( @ISA );
 package Opinion;
 use base qw( SpringfieldObject );
 
+package Credit;
+use base qw( SpringfieldObject );
+
 1;
